/* MAIN_STORE.h
   Klassendefinition fuer einen Hauptspeicher und seine Verwaltung

   Methoden
   - fetch :      liefert Referenz auf eine Speicherstelle
   - clearPage :  initialisiert einen Seitenrahmen
   - aquire :     liefert die Nummer eines freien Rahmens und reserviert ihn, blockiert bis einer frei ist
   - release :    gibt einen Rahmen frei
   - waitLowFrameNumber : wartet bis die Zahl der freien Rahmen auf den LIMIT-Wert gesunken ist
 */

#ifndef MAIN_STORE_H_
#define MAIN_STORE_H_

#include "SYNC_POOL.h"
#include "pagingInt.h"

namespace PagingNS {

  class MainStore  {
  public:
    
    typedef Byte FrameNr;  // Rahmennummern sind auf Bytegroesse beshraenkt
    
    MainStore () {}
    
    Byte &   fetch(int addr) { return ram[addr]; }
    
    void     clearPage(Byte pageNr) { for (unsigned int i = 0; i<FRAME_SIZE; ++i) ram[pageNr*FRAME_SIZE+i] = 'X'; }
    
    FrameNr  aquire()                   { return FrameNr( framePool.aquire() ); }
    void     release( FrameNr frameNr ) { framePool.release( int(frameNr) ); }
    void     waitLowFrameNumber()       { framePool.waitFull(); }
    
  private:
    SYNC_POOL<FRAME_COUNT, LIMIT_LOW> framePool;
    
    Byte   ram[FRAME_COUNT*FRAME_SIZE];
  };

}
#endif
