/* Poolverwaltung mit Synchronisation
 *
 verwaltet die Nummern freier Plaetze, die Plaetze selbst sind nicht
 Bestandteil des Pools

 Template-Parameter:
 - SIZE        : Zahl der freien Plaetze
 - LIMIT_LOW   : Zahl der freien Plaetze ab welcher der Pool als voll gilt

 Operationen:
 - aquire: liefert Nummer eines frein Platzes, blocktiert bis einer frei ist
 - release: gibt Platz frei
 - waitFull: blockiert bis der Pool voll ist (die Zahl der freien Plaetze erreicht das LIMIT)
*/

#ifndef SYNC_POOL_H_
#define SYNC_POOL_H_

#include <PMutex.h>
#include <PCondition.h>
#include <iostream>

template<int SIZE, int LIMIT_LOW>
class SYNC_POOL {
 public:
  SYNC_POOL () : zFrei(SIZE), warteFrei(&mutex), warteVoll(&mutex) {
    for (int i=0; i<SIZE; ++i)
      poolTable[i].state=free;
  }

  ~SYNC_POOL () {}

  int aquire() {
    mutex.lock();
    while (zFrei < 1) warteFrei.wait();
    //cout << "SYNC_POOL::aquire suche Rahmen" << endl;
    int i=0;
    while ( i<SIZE ) {
      if ( poolTable[i].state == free ) {
	poolTable[i].state = occupied;
	//cout << "SYNC_POOL::aquire hat Rahmen  gefunden " << i << endl;
	--zFrei;
	warteVoll.signal();
	mutex.unlock();
	return i;
      }
      ++i;
    }
    // wird (hoffentlich) nie erreicht
    std::cout << "SYSTEM-ABSTURZ: BS FEHLER " << std::endl;
    exit (1);
  }

  void release( int nr ) {
    mutex.lock();
    poolTable[nr].state=free;
    ++zFrei;
    warteFrei.signal();
    mutex.unlock();
  }

  void waitFull () {
    mutex.lock();
    while (zFrei > LIMIT_LOW) warteVoll.wait();
    mutex.unlock();
  }

 private:
  enum State { free, occupied };
  struct Descriptor {
    State  state;
  };

  Descriptor            poolTable[SIZE];

  PThreadNS::PMutex      mutex;
  int                   zFrei;        // Zahl der freien Plaetze;
  PThreadNS::PCondition       warteFrei;    // warte auf freien Platz
  PThreadNS::PCondition       warteVoll;    // warte darauf dass alles belegt ist
};

#endif
