/* SWAP_AREA.h
   Klassendefinition fuer einen Swapspace und seine Verwaltung

   Methoden
   - fetch :      liefert Referenz auf eine Speicherstelle
   - aquire :     liefert die Nummer eines freien Slots und reserviert ihn, blockiert bis einer frei ist
   - release :    gibt einen Slot frei
   - swapOut :    schreibt BYTEs ab einer uebergebenen Position in einen Slot
   - swapIn :     schreibt Inhalt eines Slots ab einer uebergebenen BYTE-Position
 */



#ifndef SWAP_AREA_H_
#define SWAP_AREA_H_

#include "SYNC_POOL.h"
#include "pagingInt.h"

namespace PagingNS {

  class SwapArea  {
  public:
    SwapArea  () {}
    
    typedef Byte SlotNr;  // Slotnummern sind auf Byte beschraenkt
    
    Byte   aquire()                  { return slotPool.aquire(); }
    void   release( SlotNr slotNr )  { slotPool.release( slotNr ); }
    
    void   swapOut( Byte *b, SlotNr d) {  // alle Bytes ab b in den Slot d
      for (unsigned int i=0; i<SLOT_SIZE; ++i) 
	swapSpace[d*SLOT_SIZE+i] = b[i];
    }
    
    void   swapIn (Byte *b, SlotNr d) {   // alle Bytes des Slots d nach b
      for (unsigned int i=0; i<SLOT_SIZE; ++i) 
	b[i] = swapSpace[d*SLOT_SIZE+i];
    } 
    
  private:
    SYNC_POOL<SLOT_COUNT, 0> slotPool;
    
    Byte   swapSpace[SLOT_COUNT*SLOT_SIZE];
  };
}
#endif
