/* VIRTUAL_MEMORY.h

  Methoden:
  - aquire        : liefert Zeiger auf eine Liste von Seitendeskriptoren (also ein Zeiger auf Seitentabelle)
  - release       : gibt Seitentabelle frei
  - throwOutPages : wirft Seite(n) raus
*/

#ifndef VIRTUAL_MEMORY_H_
#define VIRTUAL_MEMORY_H_

#include "pagingInt.h"
#include "VirtualPage.h"
#include <PMutex.h>

namespace PagingNS {

  class VirtualMemory {
  public:
    VirtualMemory () : throwOuts(0) { for (unsigned int i=0; i<MAX_PROCESS_COUNT; ++i) pageTableMgmnt[i] = free; }

    // liefert Zeiger auf eine Liste von Seitendeskriptoren (also ein Zeiger auf eine Seitentabelle)
    VirtualPage * aquire()  {
      m.lock();
      for (unsigned int i=0; i<MAX_PROCESS_COUNT; ++i)
	if ( pageTableMgmnt[i] == free ) { 
	  pageTableMgmnt[i] = occupied;
	  VirtualPage *pvp = &pageTables[i*PAGE_COUNT];
	  m.unlock();
	  return pvp; 
	}
      cout << "SYSTEM-ABSTURZ: Ueberlauf PageTabellen (zu viele Prozesse)" << endl; exit(1);
    }
    
    void release(VirtualPage *pt) {
      m.lock();
      for (unsigned int i=0; i<MAX_PROCESS_COUNT; ++i) 
	if ( &(pageTables[i*PAGE_COUNT]) == pt ) {
	  pageTableMgmnt[i] = free; 
	  for (unsigned int j=0; j<PAGE_COUNT; ++j)
	    (pt+j)->where = VirtualPage::clear;
	  m.unlock();
	  return;
	}
      cout << "SYSTEM-ABSTURZ: BS FEHLER, falsche SeitenTabelle " << hex << pt << endl; exit(1);
    }
    
    void throwOutPages () {  // wirft die Seite(n) raus
      //??
    }

    void statistic () {
      cout << "VirtualMemory Statistic\n"
	   << "ThrowOuts: " << dec << throwOuts << endl;
    }

  private:
    enum State { free, occupied };
    State             pageTableMgmnt[MAX_PROCESS_COUNT];        // Verwaltung der einzelnen Tabellen: Seite frei?
    VirtualPage       pageTables[PAGE_COUNT*MAX_PROCESS_COUNT]; // Alle SeitenDeskriptoren alle SeitenTabellen
    int               throwOuts;
    PThreadNS::PMutex  m;
  };
  

}

#endif
