/* VIRTUAL_PAGE.h
   Virtuelle Seite == Seitendeskriptor
   
   Methoden:
   - fetch : liefert Referenz auf reale Speicherstelle zu einer virtuellen in dieser Seite
*/

#ifndef VIRTUAL_PAGE_H_
#define VIRTUAL_PAGE_H_

#include "pagingInt.h"
#include "MainStore.h"
#include "SwapArea.h"

namespace PagingNS {

  class VirtualPage {
    friend class VirtualMemory;
  public:
    
    typedef Byte PageNr;      // Pagezahl und Pageoffset
    typedef Byte PageOffset;  // muessen Byteadressierbar sein
    
    VirtualPage () { where = clear; }
    
    Byte & fetch( PageOffset pageOffset ) {
      //cout << "VirtualPage::fetch("<< int(pageOffset) << ")"<<endl;
      bringIn();
      return mainStore.fetch(m*PAGE_SIZE+pageOffset);
    }
    
  private:
    enum Where {
      in,          // im Hauptspeicher
      out,         // im Swapbereich
      clear        // neu, nicht belegt
    };
    Where  where;
    Byte   m;      // wo im Speicher
    Byte   d;      // wo im SwapSpace
    
    static MainStore      &mainStore;
    static SwapArea       &swapArea;
    
    void bringIn() {
      if ( where != in ) {
	m = mainStore.aquire();   // Seitenrahmen anfordern
	if ( where == clear ) {   // Neue Seite: sauber putzen
	  mainStore.clearPage( m );
	}
	if ( where == out ) {     // Seite ist ausgelagert, d sagt wo,
	  swapArea.swapIn( &(mainStore.fetch(m*PAGE_SIZE)), d );
	  swapArea.release( d );
	}
	where = in;
      }
    }
    
    void throwOut () {
      if ( where == in ) {
	d = swapArea.aquire();    // Slot im SwapSpace anfordern
	swapArea.swapOut( &(mainStore.fetch(m*PAGE_SIZE)), d );
	mainStore.release( m );   // Rahmen freigeben
	where = out;
      }
    }
  };

}
#endif
