/*
 *  paging.cc
 *  Paging System
 */

#include "pagingInt.h"
#include "SwapArea.h"
#include "MainStore.h"
#include "VirtualPage.h"
#include "VirtualMemory.h"
#include "AddressSpace.h"


// Privater Teil des Namensraums OS_NS
//
namespace PagingNS {

/**
 *  vertauscht die bytes in w
 *  zur konvertierung von big endian nach little endian
 */
void byteSwap( Word &w )
{
  Byte t[2];
  t[0] = (reinterpret_cast<Byte *>(&w))[1];
  t[1] = (reinterpret_cast<Byte *>(&w))[0];
  w = *(reinterpret_cast<Word *>(t));
}

/**
 *  testet, ob das system big endian verwendet
 */
bool isSysBigEndian()
{
  Word w = 0x0100;      // MSB = 1, LSB = 0
  Byte *b = reinterpret_cast<Byte *>(&w);
  return b[1];            // 2-tes Byte ist 1 -> MSB ist hinten, BigEndian
}                         // 2-tes Byte ist 0 -> MSB ist vorn, LittleEndian

/**
 *  konvertiert w nach little endian
 */
void toLittleEndian( Word &w ) { // Konversion nach MSB zuerst (falls notwendig)
  if( isSysBigEndian() )
    byteSwap( w );
}


//------------------------------------------------------------------------------------------
// Realer Speicher und Swap-Bereich
//
SwapArea swapManager;
MainStore storeManager;

MainStore & VirtualPage::mainStore = storeManager;
SwapArea  & VirtualPage::swapArea  = swapManager;


//------------------------------------------------------------------------------------------
// Virtueller Speicher, virtueller Adressraum
//
VirtualMemory virtualMemoryManager;

const int AddressSpace::SIZE = 0x10000; // 2 Byte Adressraum;
VirtualMemory &AddressSpace::vm = virtualMemoryManager;

//------------------------------------------------------------------------------------------
// Paging Daemon
//
class PageDaemon : public PThreadNS::PThread
{
public:
  PageDaemon() { }

  void run()
  {
    while( true ) {
      //???
    }
  }

private:
  static VirtualMemory & vm;
  static MainStore     & rm;
};

VirtualMemory & PageDaemon::vm = virtualMemoryManager;
MainStore     & PageDaemon::rm = storeManager;

//------------------------------------------------------------------------------------------
// VMThread
//
Byte & getByteAt( LogicalAddress i, AddressSpace *as) { return as->getByteAt(i); }

VMThread::VMThread () : as(new AddressSpace) {}
VMThread::~VMThread () { delete as; }

//------------------------------------------------------------------------------------------
// Systemstart
//
class OS
{
public:
  OS () {
    //pd.start();
  }
  ~OS () {
    //pd.join();
  }
private:
  PageDaemon pd;
};

} // namespace PagingNS


//------------------------------------------------------------------------------------------
// System hochfahren
//
namespace {

PagingNS::OS os;

} // namespace
