/*
 *  Paging.h
 *  Exportierte Definitionen
 */

#include <PThread.h>
#include <sys/types.h>

#ifndef PAGING_H_
#define PAGING_H_

namespace PagingNS {

class AddressSpace;

#ifdef OS_LINUX
typedef u_int8_t   Byte;
typedef u_int16_t  Word;
#endif

#ifdef OS_SOLARIS
typedef uint8_t   Byte;
typedef uint16_t  Word;
#endif

/**
 *  logische Adresse
 */
typedef Word LogicalAddress;

/**
*  physische Adresse
*/
typedef Word PhysiscalAddress;

/**
*  gibt eine Referenz auf das Byte an der _logischen_ Adresse i
*  im Adressraum as zurck
*/
Byte &getByteAt(LogicalAddress i, AddressSpace *as);


class VMThread : public PThreadNS::PThread
{
public:
  VMThread();
  ~VMThread();

protected:
  typedef PagingNS::LogicalAddress LogicalAddress;

  /** Zugriff auf den Adressraum des Threads */
  Byte &mem(LogicalAddress i) { return getByteAt(i, as); }

private:
  /**
   *  Adressraum des Threads
   *  Zugriff nur ber mem()
   */
  AddressSpace * const as;
};

} // namespace PagingNS

#endif // PAGING_H_

